/***************************************************************************
                          sglwindow.cpp  -  description
                             -------------------
    begin                : Wed Jan 23 2002
    copyright            : (C) 2002 by Andre Stein
    email                : andre_stein@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sglwindow.h"
#include <GL/gl.h>
#include <GL/glu.h>
#include <math.h>
#include <SDL.h>


//LIGHTING PARAMETERS

const float WHOLE_SCENE_LIGHT[] = { 0.25f, 0.25f, 0.25f, 1.0f};
const float DIFFUSE_LIGHT[]  = { 0.6f, 0.6f, 0.6f, 1.0f};
const float SPECULAR_LIGHT[] = {-.1f, -0.1f, -0.1f, 1.0f};
const float LIGHT_POSITION1[] = { 0.0f, -5.0f,0.0f, 1.0f};
const float LIGHT_POSITION2[] = { 0.0f, 5.0f,0.0f, 1.0f};


const float SPOT_DIRECTION[] = {0.0f,0.0f,-1.0f};

const float MATERIAL[] = {1.0f,1.0f,1.0f,1.0f};

SGLWindow::SGLWindow()
{

}
SGLWindow::~SGLWindow()
{
}
/** No descriptions */
void SGLWindow::init()
{
	glClearColor(0.0f,0.0f,0.0f,0.0f);
	glShadeModel(GL_SMOOTH);
	glClearDepth(1.0f);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
 	glBlendFunc(GL_ONE,GL_ONE);

        //************
  	//LIGHTING
   	//************

        glLightModelfv(GL_LIGHT_MODEL_AMBIENT,WHOLE_SCENE_LIGHT);

	//Main
	glLightfv(GL_LIGHT0,GL_AMBIENT,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_SPECULAR,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT0,GL_DIFFUSE,DIFFUSE_LIGHT);
	glLightfv(GL_LIGHT0,GL_POSITION,LIGHT_POSITION1);

	glLightfv(GL_LIGHT1,GL_AMBIENT,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT1,GL_SPECULAR,SPECULAR_LIGHT);
	glLightfv(GL_LIGHT1,GL_DIFFUSE,DIFFUSE_LIGHT);
	glLightfv(GL_LIGHT1,GL_POSITION,LIGHT_POSITION2);

	//Spotlight
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, 10.0f); //60 Spotlight
	glLightf(GL_LIGHT0,GL_SPOT_EXPONENT, 60.0f);
	glLightfv(GL_LIGHT0,GL_SPOT_DIRECTION,SPOT_DIRECTION);

    	glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);

	//Materials
	glMaterialfv(GL_FRONT,GL_SPECULAR,MATERIAL);
	glMaterialf(GL_FRONT,GL_SHININESS,32.0f);

	glEnable(GL_COLOR_MATERIAL);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHT1);

        //tex gen
        initTexGen();

 	//********
  	//OBJECTS
   	//********

    	objects.init();
	spot.init();
}
/** No descriptions */
void SGLWindow::resize(int width, int height)
{
	glViewport(0,0,width,height);

	if (height == 0)
		height = 1;	

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective(45.0f,width/height,0.1f,100.0f);

	/***/
		
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}
/** No descriptions */
void SGLWindow::draw()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glLoadIdentity();

        room.draw();
	objects.draw();

	//************************
	// Projective Texture Mapping begins here
	
	spot.beginDraw(1);
	
	room.draw(true);
	objects.draw(true);

	spot.endDraw();
}

void SGLWindow::keyDown(int key)
{
	switch(key)
 	{
    	case SDLK_ESCAPE:
     		SDL_Event sent;
       		sent.type = SDL_QUIT;

         	SDL_PushEvent(&sent);
          	break;
	case SDLK_KP_PLUS:
	case SDLK_KP_MINUS:
		spot.toggleAnimation();
		break;
	case SDLK_SPACE:
		spot.switchTexture();
		break;		
        }
}

/** No descriptions */
void SGLWindow::initTexGen()
{
        //coefficients for tex gen
	//(as a matrix, these 4 vec would be an identity matrix)
	float v1[] = {1.0f,0.0f,0.0f,0.0f};
	float v2[] = {0.0f,1.0f,0.0f,0.0f};
	float v3[] = {0.0f,0.0f,1.0f,0.0f};
	float v4[] = {0.0f,0.0f,0.0f,1.0f};

	//setup coefficients for tex gen
	glTexGenfv(GL_S,GL_EYE_PLANE,v1);
	glTexGenfv(GL_T,GL_EYE_PLANE,v2);
	glTexGenfv(GL_R,GL_EYE_PLANE,v3);
	glTexGenfv(GL_Q,GL_EYE_PLANE,v4);

	//setup tex gen mode
	//it will be eye linear
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);

	//Enable tex. coord generation
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);
}
/** No descriptions */
void SGLWindow::update(unsigned int frametime)
{
        static int x, y;
        
        SDL_GetMouseState(&x,&y);
        SDL_WarpMouse(320,240);

        float moveX = float((320 - x))/100.0f;
	float moveY = float((240 - y))/100.0f;

        //Move spot according to the above mov.
	spot.move(-moveX,moveY);
	
        draw();
}
