// SSpot.cpp: Implementierung der Klasse SSpot.
//
//////////////////////////////////////////////////////////////////////

#include "sspot.h"

#include <math.h>
#include <iostream>

using namespace std;

#define ROT_SPEED 0.05f

//////////////////////////////////////////////////////////////////////
// Konstruktion/Destruktion
//////////////////////////////////////////////////////////////////////

SSpot::SSpot()
{

	currentTexture = 0;
	position[0] = 2.0f;
	position[1] = 2.0f;
	rotation	= 0.0f;
	animation	= false;
}

SSpot::~SSpot()
{

}

void SSpot::loadTextures()
{
	textures[0].createMipmap("data/smiley.bmp",GL_LINEAR_MIPMAP_NEAREST,
		GL_LINEAR,true);
	textures[1].createMipmap("data/lightmap.bmp",GL_LINEAR_MIPMAP_NEAREST,
		GL_LINEAR,true);
  	textures[2].createMipmap("data/text.bmp",GL_LINEAR_MIPMAP_NEAREST,
		GL_LINEAR,true);
}


void SSpot::init()
{
	loadTextures();
}

void SSpot::beginDraw(float frametime)
{
	//*********************************************
	// UPDATE ROTATION (if animation is turned on)

	if (animation)
	{
		rotation += ROT_SPEED*frametime;
	}

	//*********************************************

	//Update Texture Matrix accord.
	//to the spot position
	glMatrixMode(GL_TEXTURE);
		glLoadIdentity();
		/**
	         * SDL Texture correction
         	*****/
		glRotatef(180.0f,0.0f,0.0f,1.0f);
		glScalef(-1.0f,1.0f,1.0f);

		/** Other transformations **/
		glRotatef(rotation,0.0f,0.0f,1.0f);
		glTranslatef(-position[0]+2.0f,-position[1]+2.0f,0.0f);
		
		//The lightmap will be bigger
		// here: scalefactor = bigger => image = smaller
		if (currentTexture != 1)
			glScalef(2.0f,2.0f,1.0f);
		
		gluPerspective(45.0f, 1, .1, 1);
	
		//Apply transform for our spot
		glTranslatef(position[0],position[1],0.0f);
	glMatrixMode(GL_MODELVIEW);
	
	glEnable(GL_TEXTURE_2D);
	textures[currentTexture].select();
	
	//The Spot should be transparent
	glEnable(GL_BLEND);
}

void SSpot::endDraw()
{
	//Disable/Enabled everyting that has been turned
	//off/on
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

}

void SSpot::move(float x, float y)
{
	position[0] += x;
	position[1] += y;
}

void SSpot::switchTexture()
{
	currentTexture += 1;

	if (currentTexture > 2)
	{
		currentTexture = 0;
	}
}

void SSpot::dumpInfo()
{
	cout << "Dump:" << endl
 	     << "Position ," << position[0] << endl
       	     << "Size: " << position[1] << endl;
}

void SSpot::toggleAnimation()
{	
	animation = !animation;
	//cout << "Animation switched: " << animation << endl;
}
